module Syntax where

{- 

Example: (only ground inductive types)

  data D = C1 { x1 :: D1, ...,  xn ::Dn }
         | C2 ...
         ...

is printed as

  public abstract class D {}

  public class C1 extends D {
    public D1 x1;
    ...
    public Dn xn;
    public C1 (D1 x1, ..., Dn xn) {
      this.x1 = x1;
      ...
      this xn = xn;
    }
  }


  data D = C1 { x1 :: D1, ...,  xn ::Dn }
         | C2 ...
         ...
  --visitor E1 V1
  --visitor E2 V2

is printed as

  class D {
      public E1 accept (V1 v) {
          return v.visit (this);
      }
      public E2 accept (V2 v) {
          return v.visit (this);
      }
  }

  class C1 extends D {
    public D1 x1;
    ...
    public Dn xn;
    public C1 (D1 x1, ..., Dn xn) {
      this.x1 = x1;
      ...
      this xn = xn;
    }
  }

the visitor interface is created as follows

  interface V1 {
      public E1 visit (C1 x);
      public E1 visit (C2 x);
      ...
  }

Example:

  List a = Nil | Cons { head :: a, tail :: List a }

becomes

  abstract class List<a> {}

  class Nil<a> extends List<a> {
    Nil () {}
  }

  class Cons<a> extends List<a> {
    public a head;
    public List<a> tail;
    Cons (a head, List<a> tail) {
      this.head = head;
      this.tail = tail;
    }
  } 


grammar:

datadecl :: 'data' uppername '=' constructors visitors

constructor :: uppername [fields]

fields :: '{' fieldlist '}'

fieldlist :: fieldlist ',' field
           | field

field :: lowername '::' type

type :: type atom
      | atom

atom :: name
      | '[' type ']'
      | '(' type ')

visitor :: '--visitor' name name

-}

type FieldId = String
type ConstrId = String
type DataId = String
type Param = String

type ClassId = String
data TypeId = TypeId String 
            | Gen String -- type variable
              deriving (Eq, Show)

data Visitor = Visitor { name :: ClassId, returnType :: TypeId } deriving Show

data Type = List Type
          | App Type Type
          | Name String
        deriving (Eq, Show)

data Field = Field FieldId Type deriving Show
data Constructor = Constructor ConstrId [Field] deriving Show
data Data = Data DataId [Param] [Constructor] [Visitor] deriving Show

